local _, addon = ...

local initializedFlex = false
local initializedRaidFinder = false

-- I hate this
local f = CreateFrame("frame")
f:SetScript("OnEvent", function(self, event)
	addon:Setup()
end)
f:RegisterEvent("PLAYER_LOGIN")

function addon:Setup()
	-- Hook flex frame
	local function myFlexRaidFrame_OnShow(frame)
		if not initializedFlex then
			self:CreateButtons(frame, GetNumFlexRaidDungeons, GetFlexRaidDungeonInfo, FlexRaidFrame_SetRaid)
			initializedFlex = true
		end
		self:UpdateButtons(frame)
	end
	FlexRaidFrame:HookScript("OnShow", myFlexRaidFrame_OnShow)

	-- Hook raid finder frame
	local function myRaidFinderFrame_OnShow(frame)
		if not initializedRaidFinder then
			self:CreateButtons(frame, GetNumRFDungeons, GetRFDungeonInfo, RaidFinderQueueFrame_SetRaid)
			initializedRaidFinder = true
		end
		self:UpdateButtons(frame)
	end
	RaidFinderFrame:HookScript("OnShow", myRaidFinderFrame_OnShow)
end

-- Creates the buttons. LFR and Flex use different functions and dropdown menus so take them as arguments
function addon:CreateButtons(parentFrame, DungeonAmountFunc, DungeonInfoFunc, SetDropdownMenuFunc)
	local lastButton
	local buttons = {}
	-- Ok, I still don't understand anything about the positioning and sizing of stuff in WoW, but the target frame is about 280'ish tall and buttons 32 and who gives a shit about margins and aaargh I'm going crazy /headexplode
	local scale = min(280 / (DungeonAmountFunc() * 32), 1)

	for i = 1, DungeonAmountFunc() do
		local id, name = DungeonInfoFunc(i)
		local isAvailable, isAvailableToPlayer = IsLFGDungeonJoinable(id)

		if isAvailable and isAvailableToPlayer and self.raidData[id] then -- Make buttons only if we have data for the raid
			local button = CreateFrame("CheckButton", parentFrame:GetName().."BossesKilledButton"..tostring(id), parentFrame, "SpellBookSkillLineTabTemplate")
			if lastButton then
				button:SetPoint("TOPLEFT", lastButton, "BOTTOMLEFT", 0, -17)
			else
				button:SetPoint("TOPLEFT", parentFrame, "TOPRIGHT", 3, -50)
			end
			button:SetScript("OnEnter", function(self) if self.tooltip then addon:ShowTooltip(self) end end)
			button:SetScript("OnClick", function(self) SetDropdownMenuFunc(self.dungeonID) end)
			button:SetCheckedTexture(nil)
			button:Show()
			button:SetScale(scale)
			
			button.dungeonID = id
			button.dungeonName = name

			local number = button:CreateFontString(button:GetName().."Number", "OVERLAY", "NumberFont_Outline_Med")
			button.number = number
			number:SetPoint("TOPLEFT", -3, 5, "TOPLEFT")
			number:SetPoint("BOTTOMRIGHT", 7, -5, "BOTTOMRIGHT")

			table.insert(buttons, button)
			lastButton = button
		end
	end

	if #buttons == 0 then -- This might happen below level 90 I think, and at least when the next raid/expansion comes.
		print("BossesKilled: No data for any raids. This addon might need an update. Currently only MoP raids have data.")
	end

	parentFrame.BossesKilledButtons = buttons
end

function addon:UpdateButtons(parentFrame)
	local buttons = parentFrame.BossesKilledButtons

	for i = 1, #buttons do
		button = buttons[i]
		local numKilled = 0
		local tooltip = {{text = button.dungeonName}} -- Set up tooltip data with the dungeon name

		-- GetLFGDungeonNumEncounters returns the full set of bosses for the whole raid, while LFR and Flex raids
		-- are split into several separate raids, so get the amount of bosses and where to start from from our data file
		local index = self.raidData[button.dungeonID].startFrom
		local numEncounters = self.raidData[button.dungeonID].numEncounters

		for i = index, numEncounters + index - 1 do
			local bossName, _, isKilled = GetLFGDungeonEncounterInfo(button.dungeonID, i)

			if isKilled then
				table.insert(tooltip, {text = bossName, color = {RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b}})
				numKilled = numKilled + 1
			else
				table.insert(tooltip, {text = bossName, color = {GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b}})
			end
		end

		button.tooltip = tooltip

		button.number:SetText(numKilled.."/"..numEncounters)
		if numKilled == numEncounters then
			button.number:SetTextColor(0,1,0)
		else
			button.number:SetTextColor(1,0,0)
		end
	end
end

function addon:ShowTooltip(button)
	GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
	for i = 1, #button.tooltip do
		tooltip = button.tooltip[i]
		if tooltip.color then
			GameTooltip:AddLine(tooltip.text, unpack(tooltip.color))
		else
			GameTooltip:AddLine(tooltip.text)
		end
	end
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine("<Click to select this raid>")
	GameTooltip:Show()
end
